module Jackpotrising
  module Configuration
    VALID_CONNECTION_KEYS = [:endpoint, :method, :user_agent, :url].freeze
    VALID_OPTIONS_KEYS    = [:params, :api_key, :format].freeze
    VALID_CONFIG_KEYS     = VALID_CONNECTION_KEYS + VALID_OPTIONS_KEYS

    DEFAULT_ENDPOINT    = "http://api-staging.jackpotrising.com"
    DEFAULT_METHOD      = :get
    DEFAULT_USER_AGENT  = "Jackpot Rising API Ruby Gem #{Jackpotrising::VERSION}".freeze

    DEFAULT_API_KEY      = nil
    DEFAULT_FORMAT       = :json
    DEFAULT_URL          = nil

    # Build accessor methods for every config options so we can do this, for example:
    #   Awesome.format = :xml
    attr_accessor *VALID_CONFIG_KEYS

    # Make sure we have the default values set when we get 'extended'
    def self.extended(base)
      base.reset
    end

    def reset
      self.endpoint   = DEFAULT_ENDPOINT
      self.method     = DEFAULT_METHOD
      self.user_agent = DEFAULT_USER_AGENT
      self.url        = DEFAULT_URL

      self.api_key    = DEFAULT_API_KEY
      self.format     = DEFAULT_FORMAT
    end

    def configure(url, method)
      self.url = url
      self.endpoint = DEFAULT_ENDPOINT + url
      self.method = method

      yield self
    end

    # Return the configuration values set in this module
    def options
      Hash[ * VALID_CONFIG_KEYS.map { |key| [key, send(key)] }.flatten ]
    end

  end # Configuration
end
